#! /bin/bash

# MSP430 firmware version
REQ_VERSION=2.0

find_mtdnum() {
    # mtdname -> mtd number
    local mtdname=$1
    local mtd=`cat /proc/mtd | grep -F \"$mtdname\" | cut -d ':' -f 1`
    local mtdnumber="${mtd#mtd}"

    echo $mtdnumber
}

write_to_mtd() {
    # mountpoint, kind, area, pre-erase
    local dir=$1
    local kind=$2
    local area=$3
    local erase=$4

    local mtdname=

    if [ "$area" = "0" ] || [ "$area" = "1" ]; then
        mtdname=$kind.$area
    elif [ "$area" == "-1" ]; then
        mtdname=$kind
    else
        mtdname=$area
    fi

    local mtdnum=`find_mtdnum $mtdname`

    local mtdblk="/dev/mtdblock$mtdnum"
    local mmcblk=

    if [ "$mtdnum" = "" ]; then
        echo "  ! MTD \"$mtdname\" not found."
        cleanup $dir
        exit 1
    fi

    if [ "$erase" = "1" ]; then
        # MTD
        for retry in `seq 3`; do
            echo -n "  writing \"$kind\" image to /dev/mtd$mtdnum ... "
            flashcp $dir/$kind.img /dev/mtd$mtdnum &> /dev/null
            res=$?
            if [ "$res" = "0" ]; then
                break
            elif [ "$retry" -lt 3 ]; then
                echo "failed."
                echo -n "  (retry)"
            fi
        done
    fi

    if [ "$res" != "0" ]; then
        echo "failed."
        exit 1
    else
        echo "succeeded."
    fi
}

set_led() {
    # led dir, value
    local dir=$1
    local val=$2

    echo $val > $dir/brightness
}

MNTPOINT=$1
AREA=$2

DEVTYPE=$(cat /proc/device-tree/board/dev_type 2>/dev/null | tr -d '\000')

echo -n "* device type: "
if [ "${DEVTYPE}" = "HS-FS" ]; then
    echo "HS-FS"
    TIBOOT3="tiboot3.bin"
else
    echo "GP"
    TIBOOT3="tiboot3_GP.bin"
fi

if [ -f ${MNTPOINT}/${TIBOOT3}.img ]; then
    echo "* u-boot (1st stage) file found."
    write_to_mtd ${MNTPOINT} ${TIBOOT3} tiboot3 1
fi

if [ -f ${MNTPOINT}/tispl.bin.img ]; then
    echo "* u-boot (2nd stage) file found."
    write_to_mtd ${MNTPOINT} tispl.bin tispl 1
fi

# find DIO MSP430's RESET LEDs
LEDS=`find /sys/class/leds -name "MSP430_*_RST"`

${MNTPOINT}/msp430_ctrl check_version ${REQ_VERSION} &> /dev/null
if [ "$?" != "0" ]; then
    echo "* MSP430 needs to be programmed."

    for led in ${LEDS}; do
        echo "  Assert Reset: ${led##*/}"
        set_led $led 255
    done;

    ${MNTPOINT}/msp430_writer -f ${MNTPOINT}/msp430firm.txt

    for led in ${LEDS}; do
        echo "  De-Assert Reset: ${led##*/}"
        set_led $led 0
    done;

    # Set machine-type as "MA-X3xx"
    sleep 1
    echo "* Set Machine-Type as \"MA-X3xx\"."
    ${MNTPOINT}/msp430_ctrl set_machine_type
else
    echo "MSP430 is not need to update firmware."
fi

MSP430_XIO_FIRM=${MNTPOINT}/msp430dio.txt
if [ -f ${MSP430_XIO_FIRM} ]; then
    for idx in `seq 1 2`; do
        LED="/sys/class/leds/MSP430_EXT${idx}_RST"
        if [ ! -e ${LED} ]; then
            continue
        fi
        echo "* Update XIO MSP430(${idx}) firmware."
        ${MNTPOINT}/msp430_writer -f ${MSP430_XIO_FIRM} -b 2 -s EXT${idx}
    done
fi
